#!/usr/bin/env python
"""
Script to update the version in pyproject.toml from a given version tag.
This should be replaced by some sort of cli tool
"""

import sys
import toml


def update_version(version):
    """Update version in pyproject.toml file."""
    # Read the current pyproject.toml
    with open("pyproject.toml", "r") as f:
        pyproject = toml.load(f)

    print(f"Updating version to {version}")

    # Update the version in the pyproject.toml
    pyproject["project"]["version"] = version

    # Write the updated pyproject.toml
    with open("pyproject.toml", "w") as f:
        toml.dump(pyproject, f)


if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("Usage: python scripts/update_version.py VERSION")
        sys.exit(1)

    update_version(sys.argv[1])



from box_ai_agents_toolkit.box_authentication import (
    get_ccg_client,
    get_oauth_client,
    authorize_app,
)

from box_ai_agents_toolkit.box_api import (
    box_file_get_by_id,
    box_file_text_extract,
    box_file_ai_ask,
    box_file_ai_extract,
    box_file_ai_extract_structured,
    box_folder_text_representation,
    box_folder_ai_ask,
    box_folder_ai_extract,
    box_folder_ai_extract_structured,
    box_search,
    box_locate_folder_by_name,
    box_folder_list_content,
    box_file_download,
    box_available_ai_agents,
    box_claude_ai_agent_ask,
    box_claude_ai_agent_extract,
)

__all__ = [
    "get_ccg_client",
    "get_oauth_client",
    "authorize_app",
    "box_file_get_by_id",
    "box_file_text_extract",
    "box_file_ai_ask",
    "box_file_ai_extract",
    "box_file_ai_extract_structured",
    "box_folder_text_representation",
    "box_folder_ai_ask",
    "box_folder_ai_extract",
    "box_folder_ai_extract_structured",
    "box_search",
    "box_locate_folder_by_name",
    "box_folder_list_content",
    "box_file_download",
    "box_available_ai_agents",
    "box_claude_ai_agent_ask",
    "box_claude_ai_agent_extract",
]



import pytest

# from dotenv import load_dotenv
from box_sdk_gen import BoxClient
from src.box_ai_agents_toolkit import (
    get_ccg_client,
    # get_oauth_client,
)

# load_dotenv()

# @pytest.fixture
# def box_client_auth() -> BoxClient:
#     return get_oauth_client()


@pytest.fixture
def box_client_ccg() -> BoxClient:
    return get_ccg_client()



from box_sdk_gen import BoxClient, User


def test_get_ccg_client(box_client_ccg: BoxClient):
    client = box_client_ccg
    me: User = client.users.get_user_me()

    assert me.id is not None
    assert me.type == "user" or me.type == "enterprise"
    assert me.name is not None
    assert me.login is not None



from box_sdk_gen import (
    BoxClient,
    CCGConfig,
    BoxCCGAuth,
    FileWithInMemoryCacheTokenStorage,
    OAuthConfig,
    BoxOAuth,
    GetAuthorizeUrlOptions,
)
from dotenv import load_dotenv
import os
import uuid
from .box_auth_callback import open_browser, callback_handle_request

load_dotenv()
# Environment variables
CLIENT_ID = os.getenv("BOX_CLIENT_ID")
CLIENT_SECRET = os.getenv("BOX_CLIENT_SECRET")
# CCG
SUBJECT_TYPE = os.getenv("BOX_SUBJECT_TYPE")
SUBJECT_ID = os.getenv("BOX_SUBJECT_ID")
# OAuth
REDIRECT_URL = os.environ.get("BOX_REDIRECT_URL", "http://localhost:8000/callback")


def get_auth_config() -> OAuthConfig:
    return OAuthConfig(
        client_id=CLIENT_ID,
        client_secret=CLIENT_SECRET,
        token_storage=FileWithInMemoryCacheTokenStorage(".auth.oauth"),
    )


def get_ccg_config() -> CCGConfig:
    if SUBJECT_TYPE == "enterprise":
        enterprise_id = SUBJECT_ID
        user_id = None
    else:
        enterprise_id = None
        user_id = SUBJECT_ID

    return CCGConfig(
        client_id=CLIENT_ID,
        client_secret=CLIENT_SECRET,
        enterprise_id=enterprise_id,
        user_id=user_id,
        token_storage=FileWithInMemoryCacheTokenStorage(".auth.ccg"),
    )


def get_ccg_client() -> BoxClient:
    conf = get_ccg_config()
    auth = BoxCCGAuth(conf)
    return add_extra_header_to_box_client(BoxClient(auth))


def get_oauth_client() -> BoxClient:
    conf = get_auth_config()
    auth = BoxOAuth(conf)
    return add_extra_header_to_box_client(BoxClient(auth))


def add_extra_header_to_box_client(box_client: BoxClient) -> BoxClient:
    """
    Add extra headers to the Box client.

    Args:
        box_client (BoxClient): A Box client object.
        header (Dict[str, str]): A dictionary of extra headers to add to the Box client.

    Returns:
        BoxClient: A Box client object with the extra headers added.
    """
    header = {"x-box-ai-library": "mcp-server-box"}
    return box_client.with_extra_headers(extra_headers=header)


def authorize_app() -> bool:
    conf = get_auth_config()
    auth = BoxOAuth(conf)

    state = str(uuid.uuid4())
    options = GetAuthorizeUrlOptions(
        client_id=CLIENT_ID,
        redirect_uri=REDIRECT_URL,
        state=state,
    )
    auth_url = auth.get_authorize_url(options=options)
    open_browser(auth_url)
    hostname = REDIRECT_URL.split(":")[1].replace("/", "")
    port = REDIRECT_URL.split(":")[2].split("/")[0]
    callback_handle_request(auth, hostname, int(port), state)

    access_token = auth.token_storage.get()
    if not access_token:
        return False

    return True



# Copyright (c) 2024 Airbyte, Inc., all rights reserved.

import json
import logging
from dataclasses import dataclass
from typing import Iterable, List, Union
import requests

from box_sdk_gen import (
    AiAgentAsk,
    AiAgentAskTypeField,
    AiAgentBasicTextTool,
    AiAgentExtract,
    AiAgentExtractTypeField,
    AiAgentLongTextTool,
    AiExtractResponse,
    AiItemBase,
    AiItemBaseTypeField,
    AiSingleAgentResponseFull,
    BoxClient,
    BoxSDKError,
    ByteStream,
    CreateAiAskMode,
    CreateAiExtractStructuredFields,
    CreateAiExtractStructuredFieldsOptionsField,
    File,
    Folder,
    FolderMini,
    SearchForContentContentTypes,
    SearchForContentType,
)

logger = logging.getLogger(__name__)


@dataclass
class BoxFileExtended:
    file: File
    text_representation: str


def _do_request(box_client: BoxClient, url: str):
    """
    Performs a GET request to a Box API endpoint using the provided Box client.

    This is an internal helper function and should not be called directly.

    Args:
        box_client (BoxClient): An authenticated Box client object.
        url (str): The URL of the Box API endpoint to make the request to.

    Returns:
        bytes: The content of the response from the Box API.

    Raises:
        BoxSDKError: If an error occurs while retrieving the access token.
        requests.exceptions.RequestException: If the request fails (e.g., network error,
                                             4XX or 5XX status code).
    """
    try:
        access_token = box_client.auth.retrieve_token().access_token
    except BoxSDKError as e:
        raise e

    resp = requests.get(url, headers={"Authorization": f"Bearer {access_token}"})
    resp.raise_for_status()
    return resp.content


def box_file_get_by_id(client: BoxClient, file_id: str) -> File:
    return client.files.get_file_by_id(file_id=file_id)


def box_file_text_extract(client: BoxClient, file_id: str) -> str:
    # Request the file with the "extracted_text" representation hint
    file_text_representation = client.files.get_file_by_id(
        file_id,
        x_rep_hints="[extracted_text]",
        fields=["name", "representations"],
    )
    # Check if any representations exist
    if not file_text_representation.representations.entries:
        logger.debug(f"No representation for file {file_text_representation.id}")
        return ""

    # Find the "extracted_text" representation
    extracted_text_entry = next(
        (
            entry
            for entry in file_text_representation.representations.entries
            if entry.representation == "extracted_text"
        ),
        None,
    )
    if not extracted_text_entry:
        return ""

    # Handle cases where the extracted text needs generation
    if extracted_text_entry.status.state == "none":
        _do_request(extracted_text_entry.info.url)  # Trigger text generation

    # Construct the download URL and sanitize filename
    url = extracted_text_entry.content.url_template.replace("{+asset_path}", "")

    # Download and truncate the raw content
    raw_content = _do_request(client, url)

    # check to see if raw content is bytes
    if isinstance(raw_content, bytes):
        return raw_content.decode("utf-8")
    else:
        return raw_content


def box_file_ai_ask(
    client: BoxClient, file_id: str, prompt: str, ai_agent: AiAgentAsk = None
) -> str:
    mode = CreateAiAskMode.SINGLE_ITEM_QA
    ai_item = AiItemBase(id=file_id, type=AiItemBaseTypeField.FILE)
    response = client.ai.create_ai_ask(
        mode=mode, prompt=prompt, items=[ai_item], ai_agent=ai_agent
    )
    return response.answer


def box_file_ai_extract(
    client: BoxClient, file_id: str, prompt: str, ai_agent: AiAgentAsk = None
) -> dict:
    ai_item = AiItemBase(id=file_id, type=AiItemBaseTypeField.FILE)
    response = client.ai.create_ai_extract(
        prompt=prompt, items=[ai_item], ai_agent=ai_agent
    )

    # Return a dictionary from the json answer
    return json.loads(response.answer)


def box_file_ai_extract_structured(
    client: BoxClient, file_id: str, fields_json_str: str
) -> str:
    ai_item = AiItemBase(id=file_id, type=AiItemBaseTypeField.FILE)
    fields_list = json.loads(fields_json_str)
    ai_fields = []
    options = []
    for field in fields_list:
        field_options = field.get("options")
        if field_options is not None:
            for option in field.get("options"):
                options.append(
                    CreateAiExtractStructuredFieldsOptionsField(key=option.get("key"))
                )

        ai_fields.append(
            CreateAiExtractStructuredFields(
                key=field.get("key"),
                description=field.get("description"),
                display_name=field.get("display_name"),
                prompt=field.get("prompt"),
                type=field.get("type"),
                options=options if options is not None and len(options) > 0 else None,
            )
        )
    response: AiExtractResponse = client.ai.create_ai_extract_structured(
        items=[ai_item], fields=ai_fields
    )
    return json.dumps(response.to_dict(), indent=2)


def box_folder_text_representation(
    client: BoxClient,
    folder_id: str,
    is_recursive: bool = False,
    by_pass_text_extraction: bool = False,
) -> Iterable[BoxFileExtended]:
    # folder items iterator
    for item in client.folders.get_folder_items(folder_id).entries:
        if item.type == "file":
            file = box_file_get_by_id(client=client, file_id=item.id)
            if not by_pass_text_extraction:
                text_representation = box_file_text_extract(
                    client=client, file_id=item.id
                )
            else:
                text_representation = ""
            yield BoxFileExtended(file=file, text_representation=text_representation)
        elif item.type == "folder" and is_recursive:
            yield from box_folder_text_representation(
                client=client,
                folder_id=item.id,
                is_recursive=is_recursive,
                by_pass_text_extraction=by_pass_text_extraction,
            )


def box_folder_ai_ask(
    client: BoxClient,
    folder_id: str,
    prompt: str,
    is_recursive: bool = False,
    by_pass_text_extraction: bool = False,
) -> Iterable[BoxFileExtended]:
    # folder items iterator
    for item in client.folders.get_folder_items(folder_id).entries:
        if item.type == "file":
            file = box_file_get_by_id(client=client, file_id=item.id)
            if not by_pass_text_extraction:
                text_representation = box_file_ai_ask(
                    client=client, file_id=item.id, prompt=prompt
                )
            else:
                text_representation = ""
            yield BoxFileExtended(file=file, text_representation=text_representation)
        elif item.type == "folder" and is_recursive:
            yield from box_folder_ai_ask(
                client=client,
                folder_id=item.id,
                prompt=prompt,
                is_recursive=is_recursive,
                by_pass_text_extraction=by_pass_text_extraction,
            )


def box_folder_ai_extract(
    client: BoxClient,
    folder_id: str,
    prompt: str,
    is_recursive: bool = False,
    by_pass_text_extraction: bool = False,
) -> Iterable[BoxFileExtended]:
    # folder items iterator
    for item in client.folders.get_folder_items(folder_id).entries:
        if item.type == "file":
            file = box_file_get_by_id(client=client, file_id=item.id)
            if not by_pass_text_extraction:
                text_representation = box_file_ai_extract(
                    client=client, file_id=item.id, prompt=prompt
                )
            else:
                text_representation = ""
            yield BoxFileExtended(file=file, text_representation=text_representation)
        elif item.type == "folder" and is_recursive:
            yield from box_folder_ai_extract(
                client=client,
                folder_id=item.id,
                prompt=prompt,
                is_recursive=is_recursive,
                by_pass_text_extraction=by_pass_text_extraction,
            )


def box_folder_ai_extract_structured(
    client: BoxClient,
    folder_id: str,
    fields_json_str: str,
    is_recursive: bool = False,
    by_pass_text_extraction: bool = False,
) -> Iterable[BoxFileExtended]:
    # folder items iterator
    for item in client.folders.get_folder_items(folder_id).entries:
        if item.type == "file":
            file = box_file_get_by_id(client=client, file_id=item.id)
            if not by_pass_text_extraction:
                text_representation = box_file_ai_extract_structured(
                    client=client, file_id=item.id, fields_json_str=fields_json_str
                )
            else:
                text_representation = ""
            yield BoxFileExtended(file=file, text_representation=text_representation)
        elif item.type == "folder" and is_recursive:
            yield from box_folder_ai_extract_structured(
                client=client,
                folder_id=item.id,
                fields_json_str=fields_json_str,
                is_recursive=is_recursive,
                by_pass_text_extraction=by_pass_text_extraction,
            )


def box_search(
    client: BoxClient,
    query: str,
    file_extensions: List[str] | None = None,
    content_types: List[SearchForContentContentTypes] | None = None,
    ancestor_folder_ids: List[str] | None = None,
) -> List[File]:
    # content_types: List[SearchForContentContentTypes] = [
    #     SearchForContentContentTypes.NAME,
    #     SearchForContentContentTypes.DESCRIPTION,
    #     # SearchForContentContentTypes.FILE_CONTENT,
    #     SearchForContentContentTypes.COMMENTS,
    #     SearchForContentContentTypes.TAG,
    # ]
    type = [
        SearchForContentType.FILE,
    ]
    fields: List[str] = ["id", "name", "type", "size", "description"]

    search_results = client.search.search_for_content(
        query=query,
        file_extensions=file_extensions,
        ancestor_folder_ids=ancestor_folder_ids,
        content_types=content_types,
        type=type,
        fields=fields,
    )
    return search_results.entries


def box_locate_folder_by_name(
    client: BoxClient, folder_name: str, parent_folder_id: str = "0"
) -> List[Folder]:
    type = [
        SearchForContentType.FOLDER,
    ]
    fields: List[str] = ["id", "name", "type"]

    content_types: List[SearchForContentContentTypes] = [
        SearchForContentContentTypes.NAME,
    ]

    search_results = client.search.search_for_content(
        query=folder_name,
        # file_extensions=file_extensions,
        ancestor_folder_ids=parent_folder_id,
        content_types=content_types,
        type=type,
        fields=fields,
    )
    return search_results.entries


def box_folder_list_content(
    client: BoxClient, folder_id: str, is_recursive: bool = False
) -> List[Union[File, Folder]]:
    # fields = "id,name,type"
    result: List[Union[File, FolderMini]] = []

    for item in client.folders.get_folder_items(folder_id).entries:
        if item.type == "web_link":
            continue
        if item.type == "folder" and is_recursive:
            result.extend(box_folder_list_content(client, item.id, is_recursive))
        result.append(item)

    return result


def box_file_download(client: BoxClient, file_id: str) -> ByteStream:
    # file = client.files.get_file_by_id(file_id)

    return client.downloads.download_file(file_id=file_id)


def box_available_ai_agents(client: BoxClient) -> List[AiSingleAgentResponseFull]:
    return client.ai_studio.get_ai_agents().entries


def box_claude_ai_agent_ask() -> AiAgentAsk:
    return AiAgentAsk(
        type=AiAgentAskTypeField.AI_AGENT_ASK,
        long_text=AiAgentLongTextTool(
            model="aws__claude_3_7_sonnet",
        ),
        basic_text=AiAgentBasicTextTool(
            model="aws__claude_3_7_sonnet",
        ),
        long_text_multi=AiAgentLongTextTool(
            model="aws__claude_3_7_sonnet",
        ),
        basic_text_multi=AiAgentBasicTextTool(
            model="aws__claude_3_7_sonnet",
        ),
    )


def box_claude_ai_agent_extract() -> AiAgentExtract:
    return AiAgentExtract(
        type=AiAgentExtractTypeField.AI_AGENT_EXTRACT,
        long_text=AiAgentLongTextTool(
            model="aws__claude_3_7_sonnet",
        ),
        basic_text=AiAgentBasicTextTool(
            model="aws__claude_3_7_sonnet",
        ),
    )



"""Handles the call back request from Box OAuth2.0
---
This is a simple HTTP server that listens for a request from Box OAuth2.0.
picking up the code and csrf_token from the query string.
"""

import logging
import urllib.parse
import webbrowser
from http.server import BaseHTTPRequestHandler, HTTPServer

from box_sdk_gen import BoxOAuth

CSRF_TOKEN_ORIG = ""
AUTH = None


class CallbackServer(BaseHTTPRequestHandler):
    """
    Creates a mini http request handler to handle a single callback request"""

    def do_GET(self):  # pylint: disable=invalid-name
        """
        Gets the redirect call back from Box OAuth2.0
        capturing the code and csrf_token from the query string
        and calls for the completion of the OAuth2.0 process.
        """
        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.end_headers()

        params = urllib.parse.parse_qs(urllib.parse.urlparse(self.path).query)

        code = " ".join(params.get("code")) if params.get("code") else None
        state = " ".join(params.get("state")) if params.get("state") else None
        error = " ".join(params.get("error")) if params.get("error") else None
        error_description = (
            " ".join(params.get("error_description"))
            if params.get("error_description")
            else None
        )

        logging.info("code: %s", code)
        logging.info("state: %s", state)
        logging.info("error: %s", error)
        logging.info("error_description: %s", error_description)

        if state != CSRF_TOKEN_ORIG:
            error = "Invalid State token"
            error_description = (
                "The unique state token send is not the same one received."
            )

        if error:
            html_response = f"""
            <html>
                <head>
                    <title>Box Authentication Error</title>
                    <style>
                        body {{
                            font-family: Arial, sans-serif;
                            text-align: center;
                            padding-top: 50px;
                        }}
                        .container {{
                            max-width: 600px;
                            margin: 0 auto;
                            padding: 20px;
                            background-color: #f9f9f9;
                            border-radius: 5px;
                            box-shadow: 0 2px 5px rgba(0,0,0,0.1);
                        }}
                        h1 {{
                            color: #c82124;
                        }}
                        p {{
                            font-size: 16px;
                            line-height: 1.5;
                        }}
                    </style>
                </head>
                <body>
                    <div class="container">
                        <h1>Box authorization error</h1>
                        <p>An error occurred during authorization.</p>
                        <p>Error: {error}</p>
                        <p>Error Description: {error_description}</p>
                    </div>
                </body>
            </html>
            """

        if code:
            AUTH.get_tokens_authorization_code_grant(code)
            html_response = """
            <html>
                <head>
                    <title>Box Authentication Successful</title>
                    <style>
                        body {
                            font-family: Arial, sans-serif;
                            text-align: center;
                            padding-top: 50px;
                        }
                        .container {
                            max-width: 600px;
                            margin: 0 auto;
                            padding: 20px;
                            background-color: #f9f9f9;
                            border-radius: 5px;
                            box-shadow: 0 2px 5px rgba(0,0,0,0.1);
                        }
                        h1 {
                            color: #0061d5;
                        }
                        p {
                            font-size: 16px;
                            line-height: 1.5;
                        }
                    </style>
                </head>
                <body>
                    <div class="container">
                        <h1>Success!</h1>
                        <p>Authorization completed successfully.</p> 
                        <p>You can now close this window and return to the application.</p>
                    </div>
                </body>
            </html>
            """

        self.wfile.write(
            bytes(html_response, "utf-8"),
        )


def callback_handle_request(auth: BoxOAuth, hostname, port, csrf_token: str):
    """
    Handles the call back request from Box OAuth2.0
    Creates a simple HTTP server that listens for a request from Box OAuth2.0.
    """
    global CSRF_TOKEN_ORIG  # pylint: disable=global-statement
    CSRF_TOKEN_ORIG = csrf_token
    global AUTH  # pylint: disable=global-statement
    AUTH = auth

    web_server = HTTPServer((hostname, port), CallbackServer)

    logging.info(
        "Server started http://%s:%s",
        hostname,
        port,
    )

    try:
        web_server.handle_request()
    finally:
        web_server.server_close()

    logging.info("Server stopped.")


def open_browser(auth_url: str):
    """
    Opens a browser to the auth_url
    """
    webbrowser.open(auth_url)
